# -*- coding: utf-8 -*-
from functools import wraps


def hmac_required(func):
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        if not self.auth.apiKey:
            raise NameError("sign_type should be hmacsha1, so the apiKey is required")
        return func(self, *args, **kwargs)

    return wrapper


class OpsOrderLevel(object):
    """开放代理订单级别"""
    NORMAL = "dev"  # 普通
    VIP = "dev"  # vip
    SVIP = "svip"  # svip
    PRO = "ent"  # 专业版



# -*- coding: utf-8 -*-
import base64
import hashlib
import hmac


class Auth:
    """用于保存用户orderid、apiKey，以及计算签名的对象。"""

    def __init__(self, orderId, apiKey=None):
        self._orderId = orderId
        self._apiKey = apiKey

    @classmethod
    def get_string_to_sign(cls, method, endpoint, params):
        """ 生成签名原文字符串 """
        s = method + endpoint + '?'
        query_str = '&'.join("%s=%s" % (k, params[k]) for k in sorted(params))
        return s + query_str

    def sign_str(self, raw_str, method=hashlib.sha1):
        """ 生成签名串 """
        hmac_str = hmac.new(self.apiKey.encode('utf8'), raw_str.encode('utf8'), method).digest()
        return base64.b64encode(hmac_str)

    @property
    def orderId(self):
        return self._orderId

    @property
    def apiKey(self):
        return self._apiKey



# -*- coding: utf-8 -*-

from enum import Enum, unique


@unique
class EndPoint(Enum):
    """ 各个api的主机+路径 """
    GetOrderExpireTime = "dev.kdlapi.com/api/getorderexpiretime"
    GetIpWhitelist = "dev.kdlapi.com/api/getipwhitelist"
    SetIpWhitelist = "dev.kdlapi.com/api/setipwhitelist"
    GetKpsProxy = "kps.kdlapi.com/api/getkps"
    GetDpsProxy = "dps.kdlapi.com/api/getdps"
    GetOpsProxyNormalOrVip = "dev.kdlapi.com/api/getproxy"
    GetOpsProxySvip = "svip.kdlapi.com/api/getproxy"
    GetOpsProxyEnt = "ent.kdlapi.com/api/getproxy"
    CheckDpsValid = "dps.kdlapi.com/api/checkdpsvalid"
    GetIpBalance = "dps.kdlapi.com/api/getipbalance"



# -*- coding: utf-8 -*-
import json
import time
import requests

from kdl.endpoint import EndPoint
from kdl.exceptions import KdlException, KdlNameError, KdlTypeError, KdlStatusError
from kdl.utils import hmac_required, OpsOrderLevel


class Client:
    def __init__(self, auth):
        self.auth = auth

    @hmac_required
    def get_order_expire_time(self):
        """获取订单到期时间
        强制hmac验证 """
        endpoint = EndPoint.GetOrderExpireTime.value
        params = self._get_params(endpoint)
        res = self._get_base_res("GET", endpoint, params)
        return res['data']['expire_time']

    @hmac_required
    def get_ip_whitelist(self):
        """ 获取订单的ip白名单, 强制hmac验证 """
        endpoint = EndPoint.GetIpWhitelist.value
        params = self._get_params(endpoint)
        res = self._get_base_res("GET", endpoint, params)
        return res['data']['ipwhitelist']

    @hmac_required
    def set_ip_whitelist(self, iplist=None):
        """ 设置订单的ip白名单, 强制hmac验证 """
        if iplist is None:
            raise KdlNameError("miss param: iplist")
        if not (isinstance(iplist, list) or isinstance(iplist, tuple) or isinstance(iplist, str)):
            raise KdlTypeError("iplist should be a instance of list or tuple or str")
        if isinstance(iplist, list) or isinstance(iplist, tuple):
            iplist = ', '.join(iplist)
        endpoint = EndPoint.SetIpWhitelist.value
        params = self._get_params(endpoint, iplist=iplist)
        self._get_base_res("POST", endpoint, params)
        return True

    def get_dps(self, num=None, sign_type="simple", **kwargs):
        """
            获取私密代理, 默认simple鉴权
            :return 若为json格式, 则返回 data部分
            :return 若为text/xml 格式, 则原样返回
        """
        if num is None:
            raise KdlNameError("miss param: num")
        endpoint = EndPoint.GetDpsProxy.value
        params = self._get_params(endpoint, sign_type, num=num, **kwargs)
        res = self._get_base_res("GET", endpoint, params)
        if isinstance(res, dict):
            return res['data']
        return res

    @hmac_required
    def check_dps_valid(self, proxy, **kwargs):
        """
            检测私密代理有效性, 强制hmacsha1验证
            :return 返回data部分, 格式为：ip: true/false
        """
        if not proxy:
            raise KdlNameError("miss param: proxy")
        if not (isinstance(proxy, list) or isinstance(proxy, tuple) or isinstance(proxy, str)):
            raise KdlTypeError("proxy should be a instance of list or tuple or str")
        if isinstance(proxy, list) or isinstance(proxy, tuple):
            proxy = ', '.join(proxy)
        endpoint = EndPoint.CheckDpsValid.value
        params = self._get_params(endpoint, proxy=proxy)
        res = self._get_base_res("GET", endpoint, params)
        return res['data']

    @hmac_required
    def get_ip_balance(self):
        """
            获取计数版订单ip余额, 强制hmac验证
            :return 返回data中的balance字段, int类型
        """
        endpoint = EndPoint.GetIpBalance.value
        params = self._get_params(endpoint)
        res = self._get_base_res("GET", endpoint, params)
        return res['data']['balance']

    def get_kps(self, num=None, sign_type="simple", **kwargs):
        """ 获取独享代理, 默认simple鉴权 """
        if num is None:
            raise KdlNameError("miss param: num")
        endpoint = EndPoint.GetKpsProxy.value
        params = self._get_params(endpoint, sign_type, num=num, **kwargs)
        res = self._get_base_res("GET", endpoint, params)
        if isinstance(res, dict):
            return res['data']
        return res

    def get_proxy(self, num=None, sign_type="simple", order_level=OpsOrderLevel.NORMAL, **kwargs):
        """获取开放代理
        默认simple鉴权 """
        if num is None:
            raise KdlNameError("miss param: num")
        endpoint = EndPoint.GetOpsProxyNormalOrVip.value
        if order_level == OpsOrderLevel.SVIP:
            endpoint = EndPoint.GetOpsProxySvip.value
        if order_level == OpsOrderLevel.PRO:
            endpoint = EndPoint.GetOpsProxyEnt.value
        params = self._get_params(endpoint, sign_type, num=num, **kwargs)
        res = self._get_base_res("GET", endpoint, params)
        if isinstance(res, dict):
            return res['data']
        return res

    def _get_params(self, endpoint, sign_type="hmacsha1", **kwargs):
        """ 构造请求参数 """
        params = dict(timestamp=int(time.time()), sign_type=sign_type, orderid=self.auth.orderId)
        if endpoint == EndPoint.SetIpWhitelist.value:
            iplist = kwargs.get('iplist')
            params['iplist'] = iplist
            raw_str = self.auth.get_string_to_sign("POST", endpoint, params)
            params["signature"] = self.auth.sign_str(raw_str)
            return params
        elif endpoint == EndPoint.GetDpsProxy.value or endpoint == EndPoint.GetKpsProxy.value or \
                endpoint == EndPoint.GetOpsProxyNormalOrVip.value or endpoint == EndPoint.GetOpsProxySvip or \
                endpoint == EndPoint.GetOpsProxyEnt:
            params.update(kwargs)
            if sign_type == "simple":
                return params
        elif endpoint == EndPoint.CheckDpsValid.value:
            proxy = kwargs.get('proxy')
            params['proxy'] = proxy
        raw_str = self.auth.get_string_to_sign("GET", endpoint, params)
        params["signature"] = self.auth.sign_str(raw_str)
        return params

    def _get_base_res(self, method, endpoint, params):
        """ 处理基础请求, 返回请求结果dict """
        try:
            r = None
            if method == "GET":
                r = requests.get("http://" + endpoint, params=params)
            elif method == "POST":
                r = requests.post("http://" + endpoint, data=params)
            if r.status_code != 200:
                raise KdlStatusError(r.status_code, r.content.decode('utf8'))

            try:
                r = json.loads(r.content.decode('utf8'))
                code, msg = r['code'], r['msg']
                if code != 0:
                    raise KdlException(code, msg)
                return r
            except ValueError as e:
                if r.content.strip().startswith("ERROR"):
                    raise KdlException(-3, r.content)
                return r.content.decode('utf8')
        except Exception as e:
            raise e



# -*- coding: utf-8 -*-

from setuptools import setup

setup(
    name='kdl',
    version='0.0.3',
    description=(
        'kuaidaili sdk for api, site: https://www.kuaidaili.com'
    ),
    long_description=open('README.rst').read(),
    author='<kuaidaili-dev>',
    author_email='<service@kuaidaili.com>',
    license='BSD License',
    packages= [
        'kdl',
    ],
    platforms='any',
    install_requires=[
        'requests'
    ],
    url='https://github.com/kuaidaili/python-sdk/api-sdk',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries'
    ],
)


# -*- coding: utf-8 -*-

import sys


class KdlException(Exception):
    """异常类"""

    def __init__(self, code=None, message=None):
        self.code = code
        self.message = message
        self._hint_message = "[KdlException] code: {} message: {}".format(self.code, self.message)

    @property
    def hint_message(self):
        return self._hint_message

    @hint_message.setter
    def hint_message(self, value):
        self._hint_message = value

    def __str__(self):
        if sys.version_info[0] < 3 and isinstance(self.hint_message, unicode):
            return self.hint_message.encode("utf8")
        else:
            return self.hint_message


class KdlStatusError(KdlException):
    """状态码异常类"""
    def __init__(self, code, message):
        super(KdlStatusError, self).__init__(code, message)
        self.hint_message = "[KdlStatusError] status_code: {}, message: {}".format(self.code, self.message)


class KdlNameError(KdlException):
    """ 参数异常类 """
    def __init__(self, message, code=-2):
        super(KdlNameError, self).__init__(code, message)
        self.hint_message = "[KdlNameError] message: {}".format(self.message)


class KdlTypeError(KdlException):
    """ 类型异常类 """
    def __init__(self, message, code=-1):
        super(KdlTypeError, self).__init__(code, message)
        self.hint_message = "[KdlTypeError] message: {}".format(self.message)